import json
from pathlib import Path
from subprocess import PIPE, STDOUT, Popen

import webview

version = "2024-10-10"

config = {
    "page": "init",
    "branch": "slow",
    "mirror": "tuna",
}
config_path = Path("launcher.json")

try:
    with config_path.open("r") as f:
        user_config = json.load(f)
        config.update(user_config)
except Exception:
    pass

def custom_event(data):
    data = json.dumps({"log": data})
    js = f"var event = new CustomEvent('log', {{detail: {data}}}); window.dispatchEvent(event);"
    window.evaluate_js(js)


mirror_list = {
    "pypi": "https://pypi.org/simple",
    "tuna": "https://mirrors.tuna.tsinghua.edu.cn/pypi/web/simple",
    "sjtu": "https://mirror.sjtu.edu.cn/pypi/web/simple",
}


command_list = {
    "lfs": "git\\bin\\git lfs install",
    "ensurepip": "python\\python -m ensurepip --default-pip",
    "clone": "git\\bin\\git clone https://git.zhaozuohong.vip/mower-ng/mower-ng.git --branch slow",
    "fetch": lambda: f"..\\git\\bin\\git fetch origin {config['branch']}",
    "switch": lambda: f"..\\git\\bin\\git switch -f {config['branch']}",
    "reset": lambda: f"..\\git\\bin\\git reset --hard origin/{config['branch']}",
    "pip_install": lambda: f"..\\python\\Scripts\\pip install --no-cache-dir -i {mirror_list[config['mirror']]} -r requirements.txt --no-warn-script-location",
    "webview": "start ..\\python\\pythonw webview_ui.py",
    "manager": "start ..\\python\\pythonw manager.py",
}


class Api:
    def get_branch(self):
        return config["branch"]

    def set_branch(self, branch):
        config["branch"] = branch

    def get_page(self):
        return config["page"]

    def set_page(self, page):
        config["page"] = page

    def get_mirror(self):
        return config["mirror"]

    def set_mirror(self, mirror):
        config["mirror"] = mirror

    def run(self, command, cwd=None):
        command = command_list[command]
        if callable(command):
            command = command()
        custom_event(command + "\n")
        with Popen(
            command,
            stdout=PIPE,
            stderr=STDOUT,
            shell=True,
            bufsize=1,
            text=True,
            cwd=cwd,
            encoding="utf-8",
        ) as p:
            for line in p.stdout:
                custom_event(line)


window = webview.create_window(f"mower-ng launcher {version}", "ui/dist/index.html", js_api=Api())
webview.start()

with config_path.open("w") as f:
    json.dump(config, f)
